from django.db import models
from django.http import HttpResponse

from wagtail.contrib.routable_page.models import RoutablePageMixin, route
from wagtail.core.models import Page

from wagtailsharing.models import ShareableRoutablePageMixin


class RoutableTestPage(RoutablePageMixin, Page):
    text = models.TextField()

    @route(r"^$")
    def index_route(self, request):
        return HttpResponse(self.text)


class ShareableRoutableTestPage(ShareableRoutablePageMixin, Page):
    text = models.TextField()

    @route(r"^$")
    def index_route(self, request):
        return HttpResponse(self.text)



from unittest.mock import patch

from django.http import Http404, HttpResponse
from django.test import RequestFactory, TestCase

import wagtail
from wagtail.core.models import Site
from wagtail.tests.utils import WagtailTestUtils

from wagtailsharing.models import SharingSite
from wagtailsharing.tests.helpers import (
    create_draft_page,
    create_draft_routable_page,
)
from wagtailsharing.views import ServeView


def before_hook_returns_http_response(*args):
    return HttpResponse("returned by hook")


def before_hook_changes_page_title(page, request, args, kwargs):
    page.title = "hook changed title"


def after_hook_returns_http_response(page, response):
    return HttpResponse("returned by hook")


def after_hook_sets_response_header(page, response):
    response["test-hook-header"] = "foo"


class TestServeView(WagtailTestUtils, TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.default_site = Site.objects.get(is_default_site=True)

    def make_request(self, path, method="get", **kwargs):
        request = getattr(self.factory, method)(path, **kwargs)
        if wagtail.VERSION < (2, 9):
            request.site = self.default_site
        return request

    def create_sharing_site(self, hostname):
        SharingSite.objects.create(site=self.default_site, hostname=hostname)

    def assert_title_matches(self, response, title):
        self.assertContains(response, title)

    def test_no_sharing_site_exists_uses_wagtail_serve(self):
        request = self.make_request("/")
        with patch("wagtailsharing.views.wagtail_serve") as wagtail_serve:
            ServeView.as_view()(request, request.path)
            wagtail_serve.assert_called_once_with(request, request.path)

    def test_no_sharing_site_exists_post_uses_wagtail_serve(self):
        request = self.make_request("/", method="post")
        with patch("wagtailsharing.views.wagtail_serve") as wagtail_serve:
            ServeView.as_view()(request, request.path)
            wagtail_serve.assert_called_once_with(request, request.path)

    def test_sharing_site_post_uses_wagtail_serve(self):
        self.create_sharing_site(hostname="hostname")

        request = self.make_request("/", HTTP_HOST="hostname", method="post")
        with patch("wagtailsharing.views.wagtail_serve") as wagtail_serve:
            ServeView.as_view()(request, request.path)
            wagtail_serve.assert_called_once_with(request, request.path)

    def test_default_site_missing_page_raises_404(self):
        self.create_sharing_site(hostname="hostname")

        request = self.make_request("/missing/")
        with self.assertRaises(Http404):
            ServeView.as_view()(request, request.path)

    def test_sharing_site_missing_page_raises_404(self):
        self.create_sharing_site(hostname="hostname")

        request = self.make_request("/missing/", HTTP_HOST="hostname")
        with self.assertRaises(Http404):
            ServeView.as_view()(request, request.path)

    def test_default_site_unpublished_page_raises_404(self):
        self.create_sharing_site(hostname="hostname")
        create_draft_page(self.default_site, title="unpublished")

        request = self.make_request("/unpublished/")
        with self.assertRaises(Http404):
            ServeView.as_view()(request, request.path)

    def test_sharing_site_unpublished_page_returns_200(self):
        self.create_sharing_site(hostname="hostname")
        create_draft_page(self.default_site, title="draft")

        request = self.make_request("/draft/", HTTP_HOST="hostname")
        response = ServeView.as_view()(request, request.path)
        self.assertEqual(response.status_code, 200)

    def test_default_site_published_page_returns_200(self):
        self.create_sharing_site(hostname="hostname")
        page = create_draft_page(self.default_site, title="published")
        page.save_revision().publish()

        request = self.make_request("/published/")
        response = ServeView.as_view()(request, request.path)
        self.assertEqual(response.status_code, 200)

    def test_sharing_site_published_page_returns_200(self):
        self.create_sharing_site(hostname="hostname")
        page = create_draft_page(self.default_site, title="published")
        page.save_revision().publish()

        request = self.make_request("/published/", HTTP_HOST="hostname")
        response = ServeView.as_view()(request, request.path)
        self.assertEqual(response.status_code, 200)

    def test_default_site_draft_version_returns_published_version(self):
        self.create_sharing_site(hostname="hostname")
        page = create_draft_page(self.default_site, title="original")
        page.save_revision().publish()
        page.title = "changed"
        page.save_revision()

        request = self.make_request("/original/")
        response = ServeView.as_view()(request, request.path)
        self.assert_title_matches(response, "original")

    def test_sharing_site_draft_version_returns_draft_version(self):
        self.create_sharing_site(hostname="hostname")
        page = create_draft_page(self.default_site, title="original")
        page.save_revision().publish()
        page.title = "changed"
        page.save_revision()

        request = self.make_request("/original/", HTTP_HOST="hostname")
        response = ServeView.as_view()(request, request.path)
        self.assert_title_matches(response, "changed")

    def test_before_serve_page_hook_called(self):
        self.create_sharing_site(hostname="hostname")
        create_draft_page(self.default_site, title="page")

        with self.register_hook(
            "before_serve_page", before_hook_returns_http_response
        ):
            request = self.make_request("/page/", HTTP_HOST="hostname")
            response = ServeView.as_view()(request, request.path)
            self.assertContains(response, "returned by hook")

    def test_before_serve_shared_page_hook_called(self):
        self.create_sharing_site(hostname="hostname")
        create_draft_page(self.default_site, title="page")

        with self.register_hook(
            "before_serve_shared_page", before_hook_changes_page_title
        ):
            request = self.make_request("/page/", HTTP_HOST="hostname")
            response = ServeView.as_view()(request, request.path)
            self.assertContains(response, "hook changed title")

    def test_before_serve_shared_page_hook_returns_http_response(self):
        self.create_sharing_site(hostname="hostname")
        create_draft_page(self.default_site, title="page")

        with self.register_hook(
            "before_serve_shared_page", before_hook_returns_http_response
        ):
            request = self.make_request("/page/", HTTP_HOST="hostname")
            response = ServeView.as_view()(request, request.path)
            self.assertContains(response, "returned by hook")

    def test_after_serve_shared_page_hook_called(self):
        self.create_sharing_site(hostname="hostname")
        create_draft_page(self.default_site, title="page")

        with self.register_hook(
            "after_serve_shared_page", after_hook_sets_response_header
        ):
            request = self.make_request("/page/", HTTP_HOST="hostname")
            response = ServeView.as_view()(request, request.path)
            self.assertEqual(response["test-hook-header"], "foo")

    def test_after_serve_shared_page_hook_returns_http_response(self):
        self.create_sharing_site(hostname="hostname")
        create_draft_page(self.default_site, title="page")

        with self.register_hook(
            "after_serve_shared_page", after_hook_returns_http_response
        ):
            request = self.make_request("/page/", HTTP_HOST="hostname")
            response = ServeView.as_view()(request, request.path)
            self.assertContains(response, "returned by hook")

    def test_routable_page_index_route(self):
        self.create_sharing_site(hostname="hostname")
        create_draft_routable_page(self.default_site, title="routable")

        request = self.make_request("/routable/", HTTP_HOST="hostname")
        response = ServeView.as_view()(request, request.path)
        self.assertEqual(response.status_code, 200)

    def test_routable_page_sub_route(self):
        self.create_sharing_site(hostname="hostname")
        create_draft_routable_page(self.default_site, title="routable")

        request = self.make_request(
            "/routable/archive/year/2000/", HTTP_HOST="hostname"
        )
        response = ServeView.as_view()(request, request.path)
        self.assertContains(response, "ARCHIVE BY YEAR: 2000")

    def test_before_route_page_hook_called(self):
        self.create_sharing_site(hostname="sharinghostname")
        create_draft_page(self.default_site, title="page")

        with self.register_hook(
            "before_route_page", before_hook_returns_http_response
        ):
            request = self.make_request("/page/", HTTP_HOST="sharinghostname")
            response = ServeView.as_view()(request, request.path)
            self.assertContains(response, "returned by hook")



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ShareableRoutableTestAppConfig(AppConfig):
    name = "wagtailsharing.tests.shareable_routable_testapp"
    label = "test_shareable_routable_testapp"
    verbose_name = _("Shareable-Routable Page Tests")



from django.db import models

from wagtail.contrib.routable_page.models import RoutablePageMixin
from wagtail.core.models import Site


class SharingSite(models.Model):
    site = models.OneToOneField(
        Site, on_delete=models.CASCADE, related_name="sharing_site"
    )
    hostname = models.CharField(max_length=255, db_index=True)
    port = models.IntegerField(default=80)

    class Meta:
        unique_together = ("hostname", "port")

    def __str__(self):
        return self.hostname + ("" if self.port == 80 else (":%d" % self.port))

    @classmethod
    def find_for_request(cls, request):
        """
        Find a shared site associated with this HTTP request object. Raises
        Raises SharingSite.DoesNotExist if no appropriate sharing site exists.

        Uses request hostname and port to find matching sharing site.
        """
        try:
            hostname = request.get_host().split(":")[0]
        except KeyError:
            hostname = None

        try:
            port = request.get_port()
        except (AttributeError, KeyError):
            port = request.META.get("SERVER_PORT")

        return cls.objects.get(hostname=hostname, port=port)

    @property
    def root_url(self):
        if self.port == 80:
            return "http://{}".format(self.hostname)
        elif self.port == 443:
            return "https://{}".format(self.hostname)
        else:
            return "http://{}:{:d}".format(self.hostname, self.port)


class ShareableRoutablePageMixin(RoutablePageMixin):
    def route(self, request, path_components):
        if getattr(request, "routed_by_wagtail_sharing", False):
            page = self.get_latest_revision_as_page()
            # This call to RoutablePageMixin's route() is so that the  method
            # gets called with the latest-revision-as-page object as self,
            # rather than the page object that is current self in this context.
            # This ensures that, if we're being routed by wagtail sharing, we
            # serve the latest revision.
            return RoutablePageMixin.route(page, request, path_components)
        return super().route(request, path_components)



from django.conf import settings
from django.http import HttpResponse
from django.test import RequestFactory, TestCase, override_settings

from wagtail.tests.testapp.models import SimplePage

from mock import Mock, patch

from wagtailsharing.wagtail_hooks import (
    add_sharing_banner,
    add_sharing_link,
    set_routed_by_wagtail_sharing,
    set_served_by_wagtail_sharing,
)


class TestAddSharingLink(TestCase):
    def setUp(self):
        self.page = SimplePage(title="title", slug="slug", content="content")
        self.page_perms = Mock()

    def test_no_link_no_button(self):
        with patch(
            "wagtailsharing.wagtail_hooks.get_sharing_url", return_value=None
        ):
            links = add_sharing_link(self.page, self.page_perms)
            self.assertFalse(list(links))

    def test_link_makes_button(self):
        url = "http://test.domain/slug/"
        with patch(
            "wagtailsharing.wagtail_hooks.get_sharing_url", return_value=url
        ):
            links = add_sharing_link(self.page, self.page_perms)
            button = next(links)
            self.assertEqual(button.url, url)
            self.assertIn(
                self.page.get_admin_display_title(), button.attrs["title"]
            )


class TestAddSharingBanner(TestCase):
    def setUp(self):
        self.page = SimplePage(title="title", slug="slug", content="content")

    def add_banner_to_response(self, content):
        response = HttpResponse(content=content)
        add_sharing_banner(self.page, response)
        return response

    @override_settings(WAGTAILSHARING_BANNER=False)
    def test_setting_false_no_banner_added(self):
        response = self.add_banner_to_response("<body>abcde</body>")
        self.assertNotContains(response, "wagtailsharing-banner")

    @override_settings(WAGTAILSHARING_BANNER=True)
    def test_setting_true_banner_is_added(self):
        response = self.add_banner_to_response("<body>abcde</body>")
        self.assertContains(response, "wagtailsharing-banner")

    @override_settings()
    def test_setting_not_defined_defaults_to_true_banner_is_added(self):
        del settings.WAGTAILSHARING_BANNER
        response = self.add_banner_to_response("<body>abcde</body>")
        self.assertContains(response, "wagtailsharing-banner")

    def test_no_body_in_response_content_banner_not_added(self):
        response = self.add_banner_to_response("abcde")
        self.assertNotContains(response, "wagtailsharing-banner")

    def test_body_not_first_tag_still_adds_banner(self):
        content = "<html><body>abcde</body></html>"
        response = self.add_banner_to_response(content)
        self.assertContains(response, "wagtailsharing-banner")

    def test_body_tag_uppercase_still_adds_banner(self):
        response = self.add_banner_to_response("<BODY>abcde</BODY>")
        self.assertContains(response, "wagtailsharing-banner")

    def test_body_with_attributes_still_adds_banner(self):
        content = '<body foo="foo" bar="bar">abcde</body>'
        response = self.add_banner_to_response(content)
        self.assertContains(response, "wagtailsharing-banner")

    def test_banner_leaves_links_alone(self):
        content = '<body>Link <a href="#">and</a> spaces</body>'
        response = self.add_banner_to_response(content)
        self.assertContains(response, '<a href="#">and</a>')

    def test_body_has_newlines_still_adds_banner(self):
        content = """<html><body

        >
        abcde</body></html>"""
        response = self.add_banner_to_response(content)
        self.assertContains(response, "wagtailsharing-banner")


class TestSetRoutedByWagtailSharing(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_set_routed_by_wagtail_sharing(self):
        request = self.factory.get("/an-url")
        set_routed_by_wagtail_sharing(request, "")
        self.assertTrue(request.routed_by_wagtail_sharing)


class TestSetServedByWagtailSharing(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_set_served_by_wagtail_sharing(self):
        request = self.factory.get("/an-url")
        set_served_by_wagtail_sharing(None, request, [], {})
        self.assertTrue(request.served_by_wagtail_sharing)



import inspect

from django.http import Http404, HttpResponse
from django.views.generic import View

from wagtail.contrib.routable_page.models import RoutablePageMixin
from wagtail.core import hooks
from wagtail.core.url_routing import RouteResult
from wagtail.core.views import serve as wagtail_serve

from wagtailsharing.models import SharingSite


class ServeView(View):
    def dispatch(self, request, path):
        if request.method.upper() != "GET":
            return wagtail_serve(request, path)

        try:
            sharing_site = SharingSite.find_for_request(request)
        except SharingSite.DoesNotExist:
            sharing_site = None

        if not sharing_site:
            return wagtail_serve(request, path)

        # Call the before_route_page hook.
        for fn in hooks.get_hooks("before_route_page"):
            result = fn(request, path)
            if isinstance(result, HttpResponse):
                return result

        page, args, kwargs = self.route(sharing_site.site, request, path)

        return self.serve(page, request, args, kwargs)

    @staticmethod
    def route(site, request, path):
        """Retrieve a page from a site given a request and path.

        This method uses the standard `wagtail.core.Page.route` method to
        retrieve a page using its path from the given site root.

        If a requested page exists and is published, the result of `Page.route`
        can be returned directly.

        If the page exists but is not yet published, `Page.route` raises an
        `Http404`, which this method tries to catch and handle. `Page.route`
        raises `Http404` in two cases: if no page with the given path exists
        and if a page exists but is unpublished. This method catches both
        cases, and, if they fall into the latter category, returns the
        requested page back to the caller despite its draft status.
        """
        path_components = [
            component for component in path.split("/") if component
        ]

        try:
            return site.root_page.route(request, path_components)
        except Http404:
            exception_source = inspect.trace()[-1]
            stack_frame = exception_source[0]

            page = stack_frame.f_locals["self"]
            path_components = stack_frame.f_locals["path_components"]

            if isinstance(page, RoutablePageMixin):
                # This mimics the way that RoutablePageMixin uses the
                # RouteResult to store the page route view to call.
                path = "/"
                if path_components:
                    path += "/".join(path_components) + "/"

                view, args, kwargs = page.resolve_subpage(path)
                return RouteResult(page, args=(view, args, kwargs))
            elif path_components:
                raise

            return RouteResult(page)

    @staticmethod
    def serve(page, request, args, kwargs):
        # Call the before_serve_page hook.
        for fn in hooks.get_hooks("before_serve_page"):
            result = fn(page, request, args, kwargs)
            if isinstance(result, HttpResponse):
                return result

        # Get the latest revision for the requested page.
        page = page.get_latest_revision_as_page()

        # Call the before_serve_shared_page hook.
        for fn in hooks.get_hooks("before_serve_shared_page"):
            result = fn(page, request, args, kwargs)
            if isinstance(result, HttpResponse):
                return result

        # Generate the page response.
        response = page.serve(request, *args, **kwargs)

        # Call the after_serve_shared_page hook.
        for fn in hooks.get_hooks("after_serve_shared_page"):
            result = fn(page, response)
            if isinstance(result, HttpResponse):
                return result

        return response



# Generated by Django 2.2.16 on 2021-03-05 14:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0040_page_draft_title'),
        ('wagtailsharing', '0002_shareableroutablepage'),
    ]

    operations = [
        migrations.DeleteModel(
            name='ShareableRoutablePage',
        ),
    ]



from wagtail.admin import urls as wagtailadmin_urls

from wagtailsharing import urls as wagtailsharing_urls


try:
    from django.urls import include, re_path
except ImportError:
    from django.conf.urls import include, url as re_path


urlpatterns = [
    re_path(r"^admin/", include(wagtailadmin_urls)),
    re_path(r"", include(wagtailsharing_urls)),
]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _

from . import checks  # noqa F401


class WagtailSharingAppConfig(AppConfig):
    name = "wagtailsharing"
    label = "wagtailsharing"
    verbose_name = _("Wagtail Sharing")



from django.test import TestCase

from wagtail.core.models import Site
from wagtail.tests.testapp.models import SimplePage

from wagtailsharing.helpers import get_sharing_url
from wagtailsharing.models import SharingSite
from wagtailsharing.tests.helpers import create_draft_page


class TestGetSharingUrl(TestCase):
    def setUp(self):
        self.default_site = Site.objects.get(is_default_site=True)

    def create_sharing_site(self, hostname):
        SharingSite.objects.create(site=self.default_site, hostname=hostname)

    def test_unroutable_page_no_sharing_site_returns_none(self):
        page = SimplePage(title="title", slug="slug", content="content")
        self.assertIsNone(get_sharing_url(page))

    def test_unroutable_page_sharing_site_returns_none(self):
        self.create_sharing_site(hostname="hostname")
        page = SimplePage(title="title", slug="slug", content="content")
        self.assertIsNone(get_sharing_url(page))

    def test_draft_page_no_sharing_site_returns_none(self):
        page = create_draft_page(self.default_site, title="draft")
        self.assertIsNone(get_sharing_url(page))

    def test_draft_page_sharing_site_returns_url(self):
        self.create_sharing_site(hostname="hostname")
        page = create_draft_page(self.default_site, title="draft")
        self.assertEqual(get_sharing_url(page), "http://hostname/draft/")

    def test_published_page_no_sharing_site_returns_none(self):
        page = create_draft_page(self.default_site, title="published")
        page.save_revision().publish()
        self.assertIsNone(get_sharing_url(page))

    def test_published_page_sharing_site_returns_url(self):
        self.create_sharing_site(hostname="hostname")
        page = create_draft_page(self.default_site, title="published")
        page.save_revision().publish()
        self.assertEqual(get_sharing_url(page), "http://hostname/published/")



from django.utils.text import slugify

from wagtail.tests.routablepage.models import RoutablePageTest
from wagtail.tests.testapp.models import SimplePage


def create_draft_page(site, title):
    page = SimplePage(
        title=title, slug=slugify(title), content="content", live=False
    )

    site.root_page.add_child(instance=page)
    return page


def create_draft_routable_page(site, title):
    page = RoutablePageTest(title=title, live=False)
    site.root_page.add_child(instance=page)
    return page



from django.apps import apps
from django.core.checks import Error, register


@register()
def modeladmin_installed_check(app_configs, **kwargs):
    errors = []

    MODELADMIN_APP = "wagtail.contrib.modeladmin"
    if not apps.is_installed(MODELADMIN_APP):
        error_hint = "Is '{}' in settings.INSTALLED_APPS?".format(
            MODELADMIN_APP
        )

        errors.append(
            Error(
                "wagtail-sharing requires the Wagtail ModelAdmin app.",
                hint=error_hint,
                id="wagtailsharing.E001",
            )
        )

    return errors



from wagtail.core.urls import (
    serve_pattern,
    urlpatterns as wagtailcore_urlpatterns,
)

from wagtailsharing.views import ServeView


try:
    from django.urls import re_path
except ImportError:  # pragma: no cover
    from django.conf.urls import url as re_path


urlpatterns = [
    re_path(serve_pattern, ServeView.as_view(), name="wagtail_serve")
    if urlpattern.name == "wagtail_serve"
    else urlpattern
    for urlpattern in wagtailcore_urlpatterns
]



from django.db import IntegrityError
from django.test import RequestFactory, TestCase

from wagtail.core.models import Site

from wagtailsharing.models import SharingSite
from wagtailsharing.tests.shareable_routable_testapp.models import (
    RoutableTestPage,
    ShareableRoutableTestPage,
)


class TestSharingSite(TestCase):
    def setUp(self):
        self.default_site = Site.objects.get(is_default_site=True)
        self.factory = RequestFactory()

    def create(self, **kwargs):
        return SharingSite.objects.create(site=self.default_site, **kwargs)

    def test_can_create(self):
        self.create(hostname="hostname", port=8000)

    def test_site_related_name_default_empty(self):
        with self.assertRaises(SharingSite.DoesNotExist):
            self.default_site.sharing_site

    def test_site_related_name(self):
        self.create(hostname="hostname", port=8000)
        self.assertIsNotNone(self.default_site.sharing_site)

    def test_str(self):
        site = self.create(hostname="hostname", port=8000)
        self.assertEqual(str(site), "hostname:8000")

    def test_str_no_port(self):
        site = self.create(hostname="hostname")
        self.assertEqual(str(site), "hostname")

    def test_uniqueness(self):
        self.create(hostname="hostname", port=1234)
        with self.assertRaises(IntegrityError):
            self.create(hostname="hostname", port=1234)

    def test_multiple_sharing_sites_not_allowed(self):
        self.create(hostname="hostname", port=1234)
        with self.assertRaises(IntegrityError):
            self.create(hostname="otherhost", port=5678)

    def test_find_for_request(self):
        sharing_site = self.create(hostname="hostname", port=1234)
        request = self.factory.get("/", HTTP_HOST="hostname", SERVER_PORT=1234)
        self.assertEqual(SharingSite.find_for_request(request), sharing_site)

    def test_find_for_request_no_sites(self):
        request = self.factory.get("/", HTTP_HOST="hostname", SERVER_PORT=1234)
        with self.assertRaises(SharingSite.DoesNotExist):
            SharingSite.find_for_request(request)

    def test_find_for_request_wrong_port(self):
        self.create(hostname="hostname", port=1234)
        request = self.factory.get("/", HTTP_HOST="hostname", SERVER_PORT=5678)
        with self.assertRaises(SharingSite.DoesNotExist):
            SharingSite.find_for_request(request)

    def test_find_for_request_hostname_keyerror(self):
        request = self.factory.get("/", SERVER_PORT=5678)
        del request.META["SERVER_NAME"]

        with self.assertRaises(SharingSite.DoesNotExist):
            SharingSite.find_for_request(request)

    def test_find_for_request_no_server_port(self):
        request = self.factory.get("/")
        del request.META["SERVER_PORT"]

        with self.assertRaises(SharingSite.DoesNotExist):
            SharingSite.find_for_request(request)

    def test_root_url_port_80_http_no_port(self):
        site = SharingSite(hostname="test.hostname", port=80)
        self.assertEqual(site.root_url, "http://test.hostname")

    def test_root_url_port_443_https(self):
        site = SharingSite(hostname="test.hostname", port=443)
        self.assertEqual(site.root_url, "https://test.hostname")

    def test_root_url_other_port_http(self):
        site = SharingSite(hostname="test.hostname", port=1234)
        self.assertEqual(site.root_url, "http://test.hostname:1234")


class TestShareableRoutablePage(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.default_site = Site.objects.get(is_default_site=True)
        self.sharing_site = SharingSite.objects.create(
            site=self.default_site,
            hostname="sharinghostname",
            port=1234,
        )

        self.root_page = self.default_site.root_page
        self.routable_page = RoutableTestPage(
            title="Routable page", text="Published text", live=True
        )

        self.shareable_routable_page = ShareableRoutableTestPage(
            title="Shareable routable page", text="Published text", live=True
        )

        self.root_page.add_child(instance=self.routable_page)
        self.routable_page.text = "Draft text"
        self.draft_revision = self.routable_page.save_revision()

        self.root_page.add_child(instance=self.shareable_routable_page)
        self.shareable_routable_page.text = "Shareable draft text"
        self.draft_revision = self.shareable_routable_page.save_revision()

    def test_route_not_sharing(self):
        plain_response = self.client.get("/routable-page/")
        self.assertContains(plain_response, "Published text")

        shareable_response = self.client.get("/shareable-routable-page/")
        self.assertContains(shareable_response, "Published text")

    def test_route_with_sharing(self):
        # Request from the sharing site
        plain_response = self.client.get(
            "/routable-page/",
            HTTP_HOST="sharinghostname",
            SERVER_PORT=1234,
        )
        self.assertContains(plain_response, "Published text")

        shareable_response = self.client.get(
            "/shareable-routable-page/",
            HTTP_HOST="sharinghostname",
            SERVER_PORT=1234,
        )
        self.assertContains(shareable_response, "Shareable draft text")



# Generated by Django 2.2.16 on 2021-01-06 15:40

from django.db import migrations, models
import django.db.models.deletion
import wagtailsharing.models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0040_page_draft_title'),
        ('wagtailsharing', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ShareableRoutablePage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
            ],
            options={
                'abstract': False,
            },
            bases=(wagtailsharing.models.ShareableRoutablePageMixin, 'wagtailcore.page'),
        ),
    ]



# Generated by Django 2.2.17 on 2021-02-22 12:17

from django.db import migrations, models
import django.db.models.deletion
import wagtail.contrib.routable_page.models
import wagtailsharing.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0041_group_collection_permissions_verbose_name_plural'),
    ]

    operations = [
        migrations.CreateModel(
            name='RoutableTestPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('text', models.TextField()),
            ],
            options={
                'abstract': False,
            },
            bases=(wagtail.contrib.routable_page.models.RoutablePageMixin, 'wagtailcore.page'),
        ),
        migrations.CreateModel(
            name='ShareableRoutableTestPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('text', models.TextField()),
            ],
            options={
                'abstract': False,
            },
            bases=(wagtailsharing.models.ShareableRoutablePageMixin, 'wagtailcore.page'),
        ),
    ]



from django.apps import apps
from django.core.checks import Error
from django.test import SimpleTestCase, override_settings

from wagtailsharing.checks import modeladmin_installed_check


class TestModelAdminInstalledCheck(SimpleTestCase):
    @override_settings(
        INSTALLED_APPS=["wagtail.contrib.modeladmin", "wagtailsharing"]
    )
    def test_check_passes_if_modeladmin_installed(self):
        self.assertFalse(modeladmin_installed_check(apps.get_app_configs()))

    @override_settings(INSTALLED_APPS=["wagtailsharing"])
    def test_check_fails_if_modeladmin_not_installed(self):
        errors = modeladmin_installed_check(apps.get_app_configs())
        self.assertEqual(len(errors), 1)
        self.assertIsInstance(errors[0], Error)
        self.assertEqual(errors[0].id, "wagtailsharing.E001")



from importlib import reload
from unittest.mock import patch

from django.test import TestCase
from django.urls import re_path

from wagtail.core import urls as wagtail_core_urls

import wagtailsharing.urls


class TestUrlPatterns(TestCase):
    def setUp(self):
        def test_view():
            pass  # pragma: no cover

        root_patterns = [
            re_path(r"^foo/$", re_path, name="foo"),
            re_path(r"^((?:[\w\-]+/)*)$", re_path, name="wagtail_serve"),
            re_path(r"^bar/$", re_path, name="bar"),
        ]

        self.patcher = patch.object(
            wagtail_core_urls, "urlpatterns", root_patterns
        )
        self.patcher.start()
        self.addCleanup(self.patcher.stop)

        reload(wagtailsharing.urls)
        self.urlpatterns = wagtailsharing.urls.urlpatterns

    def test_leaves_previous_urls_alone(self):
        self.assertEqual(self.urlpatterns[0].name, "foo")

    def test_replaces_wagtail_serve(self):
        self.assertEqual(self.urlpatterns[1].name, "wagtail_serve")
        self.assertEqual(
            self.urlpatterns[1].callback.view_class.__name__, "ServeView"
        )

    def test_leaves_later_urls_alone(self):
        self.assertEqual(self.urlpatterns[2].name, "bar")



import re

from django.conf import settings
from django.template import loader
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _

from wagtail.admin import widgets as wagtailadmin_widgets
from wagtail.contrib.modeladmin.options import ModelAdmin, modeladmin_register
from wagtail.core import hooks

from wagtailsharing.helpers import get_sharing_url
from wagtailsharing.models import SharingSite


class SharingSiteModelAdmin(ModelAdmin):
    model = SharingSite
    menu_icon = "site"
    menu_order = 603
    add_to_settings_menu = True
    list_display = ("site", "hostname", "port")


modeladmin_register(SharingSiteModelAdmin)


@hooks.register("register_page_listing_more_buttons")
def add_sharing_link(page, page_perms, is_parent=False, next_url=None):
    sharing_url = get_sharing_url(page)

    if sharing_url:
        yield wagtailadmin_widgets.Button(
            "View sharing link",
            sharing_url,
            attrs={
                "title": _("View shared revision of '{}'").format(
                    page.get_admin_display_title()
                ),
            },
            priority=90,
        )


@hooks.register("after_serve_shared_page")
def add_sharing_banner(page, response):
    if not getattr(settings, "WAGTAILSHARING_BANNER", True):
        return

    if hasattr(response, "render") and callable(response.render):
        response.render()

    html = force_str(response.content)
    body = re.search(r"(?is)<body.*?>", html)

    if body:
        endpos = body.end()

        banner_template_name = "wagtailsharing/banner.html"
        banner_template = loader.get_template(banner_template_name)

        banner_html = banner_template.render()
        banner_html = force_str(banner_html)

        content_with_banner = html[:endpos] + banner_html + html[endpos:]
        response.content = content_with_banner


@hooks.register("before_route_page")
def set_routed_by_wagtail_sharing(request, path):
    setattr(request, "routed_by_wagtail_sharing", True)


@hooks.register("before_serve_shared_page")
def set_served_by_wagtail_sharing(page, request, args, kwargs):
    setattr(request, "served_by_wagtail_sharing", True)
